#!/usr/bin/perl

#┌─────────────────────────────────
#│ Web Forum : wforum.cgi - 2019/12/31
#│ Copyright (c) KentWeb
#│ http://www.kent-web.com/
#└─────────────────────────────────
# REBUILD(改造)
#┌─────────────────────────────────
#│  Web Forum v8.0IDP Multi : wforum.cgi - 2023/01/26
#│  Edited by TRANSFORM Since 2012
#│  https://www.transformsite.com/
#└─────────────────────────────────

# モジュール宣言
use strict;
use CGI::Carp qw(fatalsToBrowser);
use lib "./lib";
use CGI::Minimal;

# 外部ファイル取込み
require './init.cgi';
my %cf = set_init();

# データ受理
CGI::Minimal::max_read_size($cf{maxdata});
my $cgi = CGI::Minimal->new;
error('容量オーバー') if ($cgi->truncated);
my %in = parse_form($cgi);

# 認証モード
my %au = authent() if ($cf{authkey});

# 処理分岐
my $mode = $in{mode};
if ($mode eq "read") { read_log(); }
if ($mode eq "form") { form_page(); }
if ($mode eq "pwd") { pwd_form(); }
if ($mode eq "relate") { relate_log(); }
if ($mode eq "find") { find_form(); }
if ($mode eq "note") { note_page(); }
if ($mode eq "past") { past_page(); }
if ($mode eq "view") { view_page(); }

bbs_list();

#-----------------------------------------------------------
#  リスト表示
#-----------------------------------------------------------
sub bbs_list {
	# ツリー展開
	my ($list,$i) = open_tree($cf{logfile},'cur');

	# クッキー取得
	my @cook = get_cookie();
	$cook[2] ||= 'http://';

	# プルダウン作成（メール表示）
	my $op_smail = make_ops($cook[3]);

	# ページ移動ボタン
	my $pgbtn = page_btn($i,'cur');

	# カウンタ
	my $counter = bbs_count() if ($cf{counter});

	# テンプレート認識
	open(IN,"$cf{tmpldir}/list.html") or &error("open err: list.html");
	my $tmpl = join('', <IN>);
	close(IN);

	# 画像認証作成
	my ($str_plain,$str_crypt);
	if ($cf{use_captcha} > 0) {
		require $cf{captcha_pl};
		($str_plain, $str_crypt) = cap::make( $cf{captcha_key}, $cf{cap_len} );
	} else {
		$tmpl =~ s/<!-- captcha_begin -->.+<!-- captcha_end -->//s;
	}

	# 画像投稿
	if ($cf{image_upl} == 0) {
		$tmpl =~ s/<!-- image_begin -->.+<!-- image_end -->//s;
	}

	# 過去ログ
	if ($cf{pastkey} == 0) {
		$tmpl =~ s/<!-- pastlog_begin -->.+<!-- pastlog_end -->//s;
	}

	# 認証モード
	if ($cf{authkey}) {
		$tmpl =~ s/!login-name!/$au{name}/g;
	} else {
		$tmpl =~ s/<!-- auth_begin -->.+?<!-- auth_end -->//sg;
	}

	# 認証クッキー
	if ($cf{authkey} && $in{mode} eq 'login') {
		set_cookie('CGISESSID',$au{sid});
	}

	# home or logoff
	#my $home = $cf{enter_pwd} eq '' ? $cf{homepage} : "$cf{bbs_cgi}?mode=logoff";

	# プルダウン作成（検索条件）
	my %op = make_op();

	# 文字置き換え
	$tmpl =~ s/<!-- list_tree -->/$list/g;
	$tmpl =~ s/<!-- page_btn -->/$pgbtn/g;
	$tmpl =~ s/!([a-z]+_cgi|cmnurl|sub_length|nam_length|bbs_title|treehead)!/$cf{$1}/g;
	$tmpl =~ s|!bbs_css!|$cf{cmnurl}/wforum.css|g;
	$tmpl =~ s|!bbs_js!|$cf{cmnurl}/bbs.js|g;
	$tmpl =~ s/!str_crypt!/$str_crypt/g;
	$tmpl =~ s/!homepage!/$cf{homepage}/g;
	$tmpl =~ s/!new_mark!/$cf{new_mark}/g;
	$tmpl =~ s/!new_time!/$cf{new_time}/g;
	$tmpl =~ s/!treehead!/$cf{treehead_top}/g;
	$tmpl =~ s/!bbs_title!/$cf{bbs_title}/g;
	$tmpl =~ s/!word!/$in{word}/g;
	$tmpl =~ s/<!-- op_(\w+) -->/$op{$1}/g;
	$tmpl =~ s/!ptop-icon!/<img src="$cf{ico_ptop}" title="pagetop" alt="pagetop">/g;
	$tmpl =~ s|!ico:(\w+\.\w+)!|<img src="$cf{icourl}/$1">|g;
	$tmpl =~ s/!counter!/$counter/g;
	$tmpl =~ s/!q!/$in{q}/g;

	# クッキー表示
	$tmpl =~ s/!name!/$cook[0]/;
	$tmpl =~ s/!email!/$cook[1]/;
	$tmpl =~ s/!url!/$cook[2]/;
	$tmpl =~ s/<!-- op_smail -->/$op_smail/;

	# 表示
	print "Content-type: text/html; charset=utf-8\n\n";
	footer($tmpl);
}

#-----------------------------------------------------------
#  リストツリー表示
#-----------------------------------------------------------
sub open_tree {
	my ($file, $stat) = @_;

	my $param;
	if ($stat eq 'cur') {
		$param = "mode";
	} else {
		$in{pno} =~ s/\D//g;
		$param = "mode=past&pno=$in{pno}&job";
	}

	# ページ数
	$in{page} ||= 0;

	# 時間取得
	my $time = time;

	# リストタグ開始
	my $list;

	# ファイル展開
	my $i = 0;
	my $x = 0;
	my ($pad,$j,@x);
	open(IN,"$file") || &error("open err: $file");
	my $top = <IN> if ($stat eq "cur");
	while (<IN>) {
		my ($no,$reno,$lx,$sub,$email,$url,$name,$dat,$msg,$tm,$h,$pw,$w,$oya,$smail,$res,$e1,$w1,$h1,$myid,$key,$e2,$w2,$h2,$e3,$w3,$h3,$e4,$w4,$h4,$e5,$w5,$h5,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5) = split(/<>/);
		if ($reno == 0) { $i++; }
		next if ($i < $in{page} + 1);
		next if ($i > $in{page} + $cf{p_tree});

	# アイコン判断
	if ($lx == 0) {
		my $ico;
		if ($key eq '0') { $ico = 0; }
		elsif ($key eq '2') { $ico = 2; }
		elsif ($e1) { $ico = 'image'; }
		else { $ico = 1; }
		$cf{treehead} =  qq|<img src="$cf{icourl}/$cf{fld_icon}->{$ico}" border="0" style="vertical-align:middle;" title="一括\表\示" alt="一括\表\示">|;
	} else {
		my $ico;
		if ($key eq '0') { $ico = 'res_lock'; }
		elsif ($key eq '2') { $ico = 'res_lock_user'; }
		elsif ($e1) { $ico = 'res_image'; }
		else { $ico = 'res_normal'; }
		$cf{res_treehead} =  qq|<img src="$cf{icourl}/$cf{res_icon}->{$ico}" border="0" style="vertical-align:middle;">|;
	}

		$x[$lx+1]=$res;
		$x[$lx]--;

		$pad = "";
		for ($j=0; $j<$lx; $j++){
		$pad .= ($x[$j]<=0)?"　 ":"<img src=\"$cf{tree_ico}/tree_i.gif\" align=\"top\" width=\"19\" height=\"21\" border=\"0\">";
		}
		$pad .= ($x[$lx]<=0)?"<img src=\"$cf{tree_ico}/tree_f.gif\" align=\"top\" width=\"19\" height=\"21\" border=\"0\">":"<img src=\"$cf{tree_ico}/tree_e.gif\" align=\"top\" width=\"19\" height=\"21\" border=\"0\">";

		# 所定時間以内の投稿は[NEWマーク]表示
		my $newsign;
		if ($stat eq "cur" && $time - $tm < $cf{new_time} * 3600) {
			$newsign = $cf{new_mark};
		}

		# 削除記事
		if ($pw eq 'DEL') {
			if ($lx == 0) {
				if ($stat eq 'cur') {
				if ($i != $in{page} + 1) {$list .= qq||;}
					$list .= qq|<p><img src="$cf{tree_ico}/tree_b.gif" align="top" width="15" height="18"><a href="?$param=relate&no=$no&page=$in{page}">$cf{treehead}</a> - |;
				} else {
					$list .= qq|<p><img src="$cf{tree_ico}/tree_b.gif" align="top" width="15" height="18">$cf{treehead} |;
				}
				$list .= qq|$sub - <span class="tree-date">$dat</span> <font color="$cf{num_st_col}">No.$no</font><br>\n|;
			} else {
				$list .= qq|$pad|;
				$list .= qq|$cf{res_treehead} $sub - <span class="tree-date">$dat</span> <font color="$cf{num_st_col}">No.$no</font><br>\n|;
			}

		# レス記事
		} elsif ($lx != 0) {
			$list .= qq|$pad|;
			$list .= qq|$cf{res_treehead} <a href="?$param=read&no=$no&reno=$reno&oya=$oya&page=$in{page}#$no">$sub</a> - <span class="tree-date"><b>$name</b> $dat</span> <font color="$cf{num_st_col}">No.$no</font> $newsign<br>\n|;

		# 親記事
		} else {
			if ($stat eq 'cur') {
			if ($i != $in{page} + 1) {$list .= qq||;}
				$list .= qq|<p><img src="$cf{tree_ico}/tree_b.gif" align="top" width="17" height="18"><a href="?$param=relate&no=$no&page=$in{page}">$cf{treehead}</a> |;
			} else {
				$list .= qq|<p><img src="$cf{tree_ico}/tree_b.gif" align="top" width="17" height="18"><a href="?mode=past&pno=$in{pno}&job=read&oya=$oya&page=$in{page}">$cf{treehead}</a> |;
			}
			$list .= qq|<a href="?$param=read&no=$no&reno=$reno&oya=$oya&page=$in{page}#$no"> $sub</a> - <span class="tree-date"><b>$name</b> $dat</span> <font color="$cf{num_st_col}">No.$no</font> $newsign<br>\n|;
		}
	}
	close(IN);
	$list .= qq|\n|;

	# 返り値
	return ($list,$i);
}

#-----------------------------------------------------------
#  記事閲覧
#-----------------------------------------------------------
sub read_log {
	my ($flg,@new,%r,%v);
	open(IN,"$cf{logfile}") or &error("open err: $cf{logfile}");
	my $top = <IN>;
	while (<IN>) {
		chomp;
		my ($no,$re,$lx,$sub,$eml,$url,$nam,$dat,$msg,$lt,$ho,$pw,$wrap,$oya,$sml,$res,$e1,$w1,$h1,$myid,$key,$e2,$w2,$h2,$e3,$w3,$h3,$e4,$w4,$h4,$e5,$w5,$h5,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5) = split(/<>/);

		if ($in{oya} == $oya) { push(@new,$_); }
		elsif ($flg && $in{oya} != $oya) { last; }

		if ($in{no} == $no) {
			$flg = 1;
			$r{name}  = $nam;
			$r{email} = $eml;
			$r{url} = $url;
			$r{msg} = $msg;
			$r{tim} = $lt;
			$r{sub} = $sub;
			$r{myid} = $myid;
			$r{key} = $key;
			$r{sml} = $sml;
			$r{e1} = $e1;
			$r{w1} = $w1;
			$r{h1} = $h1;
			$r{e2} = $e2;
			$r{w2} = $w2;
			$r{h2} = $h2;
			$r{e3} = $e3;
			$r{w3} = $w3;
			$r{h3} = $h3;
			$r{e4} = $e4;
			$r{w4} = $w4;
			$r{h4} = $h4;
			$r{e5} = $e5;
			$r{w5} = $w5;
			$r{h5} = $h5;
			$r{imgt1} = $imgtxt1;
			$r{imgt2} = $imgtxt2;
			$r{imgt3} = $imgtxt3;
			$r{imgt4} = $imgtxt4;
			$r{imgt5} = $imgtxt5;
		}
	}
	close(IN);

	$r{msg} =~ s/\n/<br>/g;

	# レスメッセージ
	my %c;
	$c{message} = "\n&gt; $r{msg}";
	$c{message} =~ s|<br>|\n&gt; |ig;
	$c{message} = tag($c{message});
	$c{message} =~ s/<.*?>//ig;

	# レスタイトル
	if ($r{sub} =~ /^Re\^(\d+)\:(.*)/) {
		my $renum = $1 + 1;
		$c{sub} = "Re\^$renum:$2";
	}
	elsif ($r{sub} =~ /^Re\:(.*)/) { $c{sub} = "Re\^2:$1"; }
	else { $c{sub} = "Re: $r{sub}"; }

	# 自動リンク
	$r{msg} = autolink($r{msg}) if ($cf{autolink});

	# 引用部色変更
	if ($cf{ref_col}) {
		$r{msg} =~ s/([\>]|^)(&gt;[^<]*)/$1<font color="$cf{ref_col}">$2<\/font>/g;
		$r{msg} =~ s/([\>]|^)(&amp;gt;[^<]*)/$1<font color="$cf{ref_col}">$2<\/font>/g;
	}

	$r{msg} = tag($r{msg});

	# 画像付加
	$r{msg} = put_image1($in{no},$r{sub},$r{name},$r{msg},$r{e1},$r{w1},$r{h1},$r{imgt1}) if ($r{e1});
	$r{msg} = put_image2($in{no},$r{sub},$r{name},$r{msg},$r{e2},$r{w2},$r{h2},$r{imgt2}) if ($r{e2});
	$r{msg} = put_image3($in{no},$r{sub},$r{name},$r{msg},$r{e3},$r{w3},$r{h3},$r{imgt3}) if ($r{e3});
	$r{msg} = put_image4($in{no},$r{sub},$r{name},$r{msg},$r{e4},$r{w4},$r{h4},$r{imgt4}) if ($r{e4});
	$r{msg} = put_image5($in{no},$r{sub},$r{name},$r{msg},$r{e5},$r{w5},$r{h5},$r{imgt5}) if ($r{e5});

	# 投稿日時
	$r{date} = get_time($r{tim});

	# e-mail表記
	if ($r{email} && $r{sml} eq '0') {
		$r{email} = qq|&nbsp; &lt;<a href="mailto:$r{email}" class="num">$r{email}</a>&gt;|;
	} else {
		$r{email} = '';
	}

	# URL表記
	if ($r{url}) {
		if ($r{url}) { $r{url} = qq|<a href="$r{url}" target="_top" class="num">$r{url}</a>|; }
	}

	# 関連ツリー作成
	my $tree;
	my $x = 0;
	my $x = 0;
	my ($pad,$j,@x);
	foreach (@new) {
		my ($no,$re,$lx,$sub,$eml,$url,$nam,$dat,$msg,$lt,$ho,$pw,$wrap,$oya,$sml,$res,$e1,$w1,$h1,$myid,$key,$e2,$w2,$h2,$e3,$w3,$h3,$e4,$w4,$h4,$e5,$w5,$h5,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5) = split(/<>/);

	# アイコン判断
	if ($lx != 0) {
		my $ico;
		if ($key eq '0') { $ico = 'res_lock'; }
		elsif ($key eq '2') { $ico = 'res_lock_user'; }
		elsif ($e1) { $ico = 'res_image'; }
		else { $ico = 'res_normal'; }
		$cf{res_treehead} =  qq|<img src="$cf{icourl}/$cf{res_icon}->{$ico}" border="0" style="vertical-align:middle;">|;
	} else {
		my $ico;
		if ($key eq '0') { $ico = 0; }
		elsif ($key eq '2') { $ico = 2; }
		elsif ($e1) { $ico = 'image'; }
		else { $ico = 1; }
		$cf{treehead} =  qq|<img src="$cf{icourl}/$cf{fld_icon}->{$ico}" border="0" style="vertical-align:middle;" title="一括\表\示" alt="一括\表\示">|;
	}

		$x[$lx+1]=$res;
		$x[$lx]--;
		$pad = "";
		for ($j=0; $j<$lx; $j++){
		$pad .= ($x[$j]<=0)?"　 ":"<img src=\"$cf{tree_ico}/tree_i.gif\" align=\"top\" width=\"19\" height=\"21\" border=\"0\">";
		}
		$pad .= ($x[$lx]<=0)?"<img src=\"$cf{tree_ico}/tree_f.gif\" align=\"top\" width=\"19\" height=\"21\" border=\"0\">":"<img src=\"$cf{tree_ico}/tree_e.gif\" align=\"top\" width=\"19\" height=\"21\" border=\"0\">";

		if ($lx != 0) {
			$tree .= qq|$pad|;
			$tree .= qq|$cf{res_treehead} <a href="$cf{bbs_cgi}?no=$no&reno=$re&oya=$oya&mode=read&page=$in{page}">$sub</a> - <span class="tree-date">$dat</span> |;
		} else {
			$tree .= qq|<img src="$cf{tree_ico}/tree_b.gif" align="top" width="17" height="17"><a href="$cf{bbs_cgi}?mode=relate&no=$no&page=$in{page}">$cf{treehead}</a> <a href="$cf{bbs_cgi}?no=$no&reno=$re&oya=$oya&mode=read&page=$in{page}">$sub</a> - <span class="tree-date"><B>$nam</B> $dat</span>|;
		}

		if ($in{no} == $no) {
			$tree .= qq| <b style="color:$cf{num_em_col}">No.$no</b><br>\n|;
		} else {
			$tree .= qq| <span style="color:$cf{num_st_col}">No.$no</span><br>\n|;
		}
	}
	$tree .= qq|\n|;

	# クッキー取得
	#($c{name},$c{email},$c{url},$c{smail},$c{ref_no}) = ref_get_cookie();
	#$c{url} ||= 'http://';
	#my $ref_no = "$c{ref_no}";

	# クッキー取得
	my @cook = refres_get_cookie();
	$cook[2] ||= 'http://';
	my $ref_no = $cook[4];

	# 添付ファイル数
	my ($op_attach,$attach,$extxt,$att_msg,$gfc,$refno);

	$ref_no = $ref_no ne '' ? $ref_no : $cf{attach_defo};

	if ($ref_no > 0) {
		foreach $gfc (1 .. $ref_no) {
			$attach .= qq|<div id="attachment">添付 $gfc : <label><input type="file" name="upfile$gfc" id="upload$gfc" class="fileinput">ファイルを添付する</label><span class="filename">選択されていません</span>&nbsp;<input type="text" name="imgtext$gfc" class="reg-imgtxt" placeholder="添付 $gfc タイトル" value="" size="40"></div>\n|;
		}
	}

	if ($cf{add_usermax} > 0){
		foreach (@{$cf{addattach}}) {

			$refno = $_;
			$refno = "　　" . $refno;
			$refno =~ s/0/０/g;
			$refno =~ s/1/１/g;
			$refno =~ s/2/２/g;
			$refno =~ s/3/３/g;
			$refno =~ s/4/４/g;
			$refno =~ s/5/５/g;

			if ($ref_no == $_) {
				$op_attach .= qq|<option value="$_">$refno個</option>\n|;
			} else {
				$op_attach .= qq|<option value="$_">$refno個</option>\n|;
			}
		}
	}

		if ($ref_no == 0) { 
			$att_msg .= qq|<small><span style="color:#00885a">参照フォームを追加出来ます</span> <img src="./cmn/arrow-gfno.png" alt="追加" class="icon2"></small>\n|;
		} elsif ($ref_no == 5) { 
			$att_msg .= qq|<small><span style="color:#d83473">参照フォームは５個が限度です</span> <img src="./cmn/arrow-gfno01.png" alt="追加" class="icon2"></small>\n|;
		} else { 
			$att_msg .= qq|<small><span style="color:#00885a">参照フォームの数を変更できます</span> <img src="./cmn/arrow-gfno.png" alt="追加" class="icon2"></small>\n|;
		}

	# プルダウン作成（メール表示）
	my $op_smail = make_ops($cook[3]);

	# 記事番号
	$c{num} = $in{no};
	$c{oya} = $in{oya};

	# 返信フォーム
	my $resfm = 1;
	if ($r{key} != 1) {
		$resfm = 0;
	}

	# テンプレート認識
	open(IN,"$cf{tmpldir}/read.html") or &error("open err: read.html");
	my $tmpl = join('', <IN>);
	close(IN);

	# 画像認証作成
	my ($str_plain,$str_crypt);
	if ($cf{use_captcha} > 0) {
		require $cf{captcha_pl};
		($str_plain, $str_crypt) = cap::make( $cf{captcha_key}, $cf{cap_len} );
	} else {
		$tmpl =~ s/<!-- captcha_begin -->.+<!-- captcha_end -->//s;
	}

	if (!$r{url}) {
		$tmpl =~ s/<!-- url_begin -->.+<!-- url_end -->//s;
	}

	# 画像投稿
	if (!$cf{image_upl}) {
		$tmpl =~ s/<!-- image_begin -->.+<!-- image_end -->//s;
	}

	# 過去ログ
	if ($cf{pastkey} == 0) {
		$tmpl =~ s/<!-- pastlog_begin -->.+<!-- pastlog_end -->//s;
	}

	# ロック/solved時返信ボタン
	if ($r{key} == 0) {
		$tmpl =~ s/<!-- lock_begin -->.+?<!-- lock_end -->//s;
	} elsif ($r{key} == 2)  {
		$tmpl =~ s/<!-- lock_begin -->.+?<!-- lock_end -->//s;
	}

	# 認証モード
	if ($cf{authkey}) {
		$tmpl =~ s/!login-name!/$au{name}/g;
	} else {
		$tmpl =~ s/<!-- id_begin -->.+?<!-- id_end -->//s;
	}
	if (!$r{myid}) {
		$tmpl =~ s/<!-- id_begin -->.+?<!-- id_end -->//s;
	}

	# 編集ボタン
	$tmpl =~ s|!ico_edit!|<a href="?mode=pwd&no=$c{num}"><img src="$cf{icourl}/edit.png" alt="記事編集"></a>|g;

	# ロックコメント
	if ($r{key} == 0) {
		$tmpl =~ s|<!-- lock_com -->|<div align="right"><div id="lock"><font color="#8080c0">このスレッドは管理者によって</font><font color="red"><b>ロック</b></font><font color="#8080c0">されていますので、返信並びに編集は出来ません！</font></div></div>|g;
	} elsif ($r{key} == 2)  {
		$tmpl =~ s|<!-- solved_com -->|<div align="right"><div id="solved"><font color="#8080c0">このスレッドは</font><font color="red"><b>解決済</b></font><font color="#8080c0">ですので、返信並びに編集は出来ません！</font></div></div>|g;
	}

	# 返信フォーム/編集ボタン
	if (!$resfm) {
		$tmpl =~ s/<!-- resform_begin -->.+<!-- resform_end -->//sg;
		$tmpl =~ s/<!-- edit_begin -->.+?<!-- edit_end -->//sg;
	}

	if (!$resfm) {
		$tmpl =~ s/<!-- reply_begin -->.+?<!-- reply_end -->//s;
	}

	# 文字置き換え
	$tmpl =~ s/!bbs_title!/$cf{bbs_title}/g;
	$tmpl =~ s|!bbs_css!|$cf{cmnurl}/wforum.css|g;
	$tmpl =~ s|!bbs_js!|$cf{cmnurl}/bbs.js|g;
	$tmpl =~ s/!treehead!/$cf{treehead}/g;
	$tmpl =~ s/!([a-z]+_cgi)!/$cf{$1}/g;
	$tmpl =~ s/!bbs_cgi!/$cf{bbs_cgi}/g;
	$tmpl =~ s/!post_cgi!/$cf{post_cgi}/g;
	$tmpl =~ s/<!-- tree -->/$tree/g;
	$tmpl =~ s/!str_crypt!/$str_crypt/g;
	$tmpl =~ s/!r_(\w+)!/$r{$1}/g;
	$tmpl =~ s/!(\w+)!/$c{$1}/g;
	$tmpl =~ s|!ico:(\w+\.\w+)!|<img src="$cf{icourl}/$1" class="icon">|g;
	$tmpl =~ s/!reg-name!/$cook[0]/;
	$tmpl =~ s/!reg-email!/$cook[1]/;
	$tmpl =~ s/!reg-url!/$cook[2]/;
	$tmpl =~ s/<!-- op_smail -->/$op_smail/;
	$tmpl =~ s/<!-- attach -->/$attach/g;
	$tmpl =~ s/<!-- add_attach -->/$op_attach/g;
	$tmpl =~ s/!reg-attmsg!/$att_msg/g;

		$ref_no =~ s/0/０/g;
		$ref_no =~ s/1/１/g;
		$ref_no =~ s/2/２/g;
		$ref_no =~ s/3/３/g;
		$ref_no =~ s/4/４/g;
		$ref_no =~ s/5/５/g;

	$tmpl =~ s/!reg-refno!/$ref_no/g;

	# 表示
	print "Content-type: text/html; charset=utf-8\n\n";
	footer($tmpl);
}

#-----------------------------------------------------------
#  タグ復元
#-----------------------------------------------------------
sub tag {
	local($_) = @_;

	# 変換
	s/&lt;/</g;
	s/&gt;/>/g;
	s/&amp;/&/g;
	s/&quot;/"/g;
	$_;
}

#-----------------------------------------------------------
#  一括表示
#-----------------------------------------------------------
sub relate_log {
	my ($obi_ttl,$tree,$count);

	if ($in{job} eq "new") {
		$obi_ttl = '新着表示';
	} else {
		$obi_ttl = '一括表示';
		$tree .= "<ul style=\"LIST-STYLE-IMAGE:url('./ico/folder_page.png')\">\n";
	}

	# 親記事を出力
	my $x = 0;
	my ($pad,$j,@x);
	my ($flg,@data,@sort,@file);
	open(IN,"$cf{logfile}") or &error("open err: $cf{logfile}");
	my $top = <IN>;
	while (<IN>) {
		chomp;
		my ($no,$re,$lx,$sub,$eml,$url,$nam,$dat,$msg,$time,$ho,$pw,$wrap,$oya,$sml,$res,$e1,$w1,$h1,$myid,$key,$e2,$w2,$h2,$e3,$w3,$h3,$e4,$w4,$h4,$e5,$w5,$h5,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5) = split(/<>/);

	# アイコン判断
	if ($in{no} == $no) {
		my $ico;
		if ($key eq '0') { $ico = 0; }
		elsif ($key eq '2') { $ico = 2; }
		elsif ($e1) { $ico = 'image'; }
		else { $ico = 1; }
		$cf{treehead} =  qq|<img src="$cf{icourl}/$cf{fld_icon}->{$ico}" border="0" style="vertical-align:middle;" title="一括\表\示" alt="一括\表\示">|;
	} else {
		my $ico;
		if ($key eq '0') { $ico = 'res_lock'; }
		elsif ($key eq '2') { $ico = 'res_lock_user'; }
		elsif ($e1) { $ico = 'res_image'; }
		else { $ico = 'res_normal'; }
		$cf{res_treehead} =  qq|<img src="$cf{icourl}/$cf{res_icon}->{$ico}" border="0" style="vertical-align:middle;">|;
	}

		$x[$lx+1]=$res;
		$x[$lx]--;

		$pad = "";
		for ($j=0; $j<$lx; $j++){
		$pad .= ($x[$j]<=0)?"　 ":"<img src=\"$cf{tree_ico}/tree_i.gif\" align=\"top\" width=\"19\" height=\"22\" border=\"0\">";
		}
		$pad .= ($x[$lx]<=0)?"<img src=\"$cf{tree_ico}/tree_f.gif\" align=\"top\" width=\"17\" height=\"18\" border=\"0\">":"<img src=\"$cf{tree_ico}/tree_e.gif\" align=\"top\" width=\"17\" height=\"21\" border=\"0\">";

		## 新着一括のとき
		if ($in{job} eq "new") {
			push(@file,$_);
			push(@sort,$time);

		## ツリー一括のとき
		} else {
			# 関連記事スタート
			if ($in{no} == $oya) {
				$flg = 1;
				push(@data,$_);

		if ($lx != 0) {
			$tree .= qq|$pad|;
			$tree .= qq|$cf{res_treehead} <a href=\"#$no\">$sub</a> - <B>$nam</B> <span class="tree-date">$dat</span> <font color=\"$cf{num_st_col}\">No.$no</font><br>\n|;
		} else {
			$tree .= qq|<img src="$cf{tree_ico}/tree_b.gif" align="top" width="17" height="18"><a href="?mode=relate&no=$no&page=$in{page}">$cf{treehead}</a> - <a href="?no=$no&reno=$re&oya=$oya&mode=read&page=$in{page}">$sub</a> - <B>$nam</B> <span class="tree-date">$dat</span><font color=\"$cf{num_st_col}\">No.$no</font><br>\n|;
		}

			# 関連記事完了
			} elsif ($flg && $in{no} != $oya) {
				last;
			}
		}
	}
	close(IN);

	# 移動ボタン変数
	my $pgbtn;

	# 新着
	if ($in{job} eq "new") {
		my $i;
		foreach ( @file[ sort{ $sort[$b] <=> $sort[$a] } 0 .. $#sort ] ) {
			$i++;
			next if ($i < $in{page} + 1);
			next if ($i > $in{page} + $cf{new_pgbtn});

			push(@data,$_);
		}

		# ページ移動ボタン
		$pgbtn = new_pgbtn($i);

	# ツリー一括
	} else {
		while ($x > 0) { $tree .= "\n"; $x--; }
		$tree .= qq|\n|;
	}

	# テンプレート認識
	open(IN,"$cf{tmpldir}/relate.html") or &error("open err: relate.html");
	my $tmpl = join('', <IN>);
	close(IN);

	$tmpl =~ s|!bbs_css!|$cf{cmnurl}/wforum.css|g;
	$tmpl =~ s|!bbs_js!|$cf{cmnurl}/bbs.js|g;
	$tmpl =~ s/!ptop-icon!/<img src="$cf{ico_ptop}" title="pagetop" alt="pagetop">/g;

	# テンプレート分割
	my ($head,$loop,$foot) = $tmpl =~ /(.+)<!-- msg_begin -->(.+)<!-- msg_end -->(.+)/s
			? ($1,$2,$3) : &error("テンプレートが不正です");

	# 文字置き換え
	foreach ($head,$foot) {
		s/!obi_ttl!/$obi_ttl/g;
		s/!bbs_title!/$cf{bbs_title}/g;
		s/<!-- tree -->/$tree/g;
		s/!bbs_cgi!/$cf{bbs_cgi}/g;
		s/!page_btn!/$pgbtn/g;
	}

	# 画面展開
	print "Content-type: text/html; charset=utf-8\n\n";
	print $head;

	foreach (@data) {
		my ($no,$reno,$lx,$sub,$eml,$url,$name,$dat,$msg,$t,$ho,$pw,$wrap,$oya,$sml,$res,$e1,$w1,$h1,$myid,$key,$e2,$w2,$h2,$e3,$w3,$h3,$e4,$w4,$h4,$e5,$w5,$h5,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5) = split(/<>/);
		next if ($pw eq 'DEL');

		# 自動リンク
		$msg = autolink($msg) if ($cf{autolink});

		# 引用部色変更
		if ($cf{ref_col}) {
			$msg =~ s/([\>]|^)(&gt;[^<]*)/$1<font color="$cf{ref_col}">$2<\/font>/g;
			$msg =~ s/([\>]|^)(&amp;gt;[^<]*)/$1<font color="$cf{ref_col}">$2<\/font>/g;
		}

		$msg = tag($msg);

		# 画像付加
		$msg = put_image1($no,$sub,$name,$msg,$e1,$w1,$h1,$imgtxt1) if ($e1);
		$msg = put_image2($no,$sub,$name,$msg,$e2,$w2,$h2,$imgtxt2) if ($e2);
		$msg = put_image3($no,$sub,$name,$msg,$e3,$w3,$h3,$imgtxt3) if ($e3);
		$msg = put_image4($no,$sub,$name,$msg,$e4,$w4,$h4,$imgtxt4) if ($e4);
		$msg = put_image5($no,$sub,$name,$msg,$e5,$w5,$h5,$imgtxt5) if ($e5);

		# 日付
		my $date = get_time($t);

		# e-mail
		if ($eml && $sml eq '0') { $name = qq|<a href="mailto:$eml">$name</a>|; }

		# URL
		$url &&= qq|<a href="$url" target="_blank"><img src="$cf{ico_home}" class="icon"></a>|;

		# 返信フォーム
		my $resfm = 1;
		if ($key != 1) {
			$resfm = 0;
		}

		my $tmp = $loop;
		$tmp =~ s/!sub!/$sub/g;
		$tmp =~ s/!num!/$no/g;
		$tmp =~ s/!date!/$date/g;
		$tmp =~ s/!name!/$name/g;
		$tmp =~ s/!url!/$url/g;
		$tmp =~ s/!msg!/$msg/g;
		$tmp =~ s/!bbs_cgi!/$cf{bbs_cgi}/g;
		$tmp =~ s/!reno!/$reno/g;
		$tmp =~ s/!oya!/$oya/g;
		$tmp =~ s|!ico:(\w+\.\w+)!|<img src="$cf{icourl}/$1">|g;

	# 認証モード
	$tmp =~ s/<!-- id -->/[ID:$myid]/g if ($cf{authkey} && $myid);

	if (!$url) {
		$tmp =~ s/<!-- url_begin -->.+?<!-- url_end -->//sg;
	}

	# ロックコメント
	if ($key == 0) {
		$tmp =~ s|<!-- lock_com -->|<div align="right"><div id="lock"><font color="#8080c0">このスレッドは管理者によって</font><font color="red"><b>ロック</b></font><font color="#8080c0">されていますので、返信並びに編集は出来ません！</font></div></div>|g;
	} elsif ($key == 2)  {
		$tmp =~ s|<!-- solved_com -->|<div align="right"><div id="solved"><font color="#8080c0">このスレッドは</font><font color="red"><b>解決済</b></font><font color="#8080c0">ですので、返信並びに編集は出来ません！</font></div></div>|g;
	}

		# 返信ボタン
		if (!$resfm) {
			$tmp =~ s/<!-- edit_begin -->.+?<!-- edit_end -->//sg;
		}
		print $tmp;
	}

	# フッタ
	footer($foot);
}

#-----------------------------------------------------------
#  投稿フォーム
#-----------------------------------------------------------
sub form_page {

	# クッキー取得
	my @cook = ref_get_cookie();
	$cook[2] ||= 'http://';
	my $ref_no = $cook[4];

	# 添付ファイル数
	my ($op_attach,$attach,$extxt,$att_msg,$gfc,$refno);

	$ref_no = $ref_no ne '' ? $ref_no : $cf{attach_defo};

	if ($ref_no > 0) {
		foreach $gfc (1 .. $ref_no) {
			$attach .= qq|<div id="attachment">添付 $gfc : <label><input type="file" name="upfile$gfc" id="upload$gfc" class="fileinput">ファイルを添付する</label><span class="filename">選択されていません</span>&nbsp;<input type="text" name="imgtext$gfc" class="reg-imgtxt" placeholder="添付 $gfc タイトル" value="" size="40"></div>\n|;
		}
	}

	if ($cf{add_usermax} > 0){
		foreach (@{$cf{addattach}}) {

			$refno = $_;
			$refno = "　　" . $refno;
			$refno =~ s/0/０/g;
			$refno =~ s/1/１/g;
			$refno =~ s/2/２/g;
			$refno =~ s/3/３/g;
			$refno =~ s/4/４/g;
			$refno =~ s/5/５/g;

			if ($ref_no == $_) {
				$op_attach .= qq|<option value="$_">$refno個</option>\n|;
			} else {
				$op_attach .= qq|<option value="$_">$refno個</option>\n|;
			}
		}
	}

		if ($ref_no == 0) { 
			$att_msg .= qq|<small><span style="color:#00885a">参照フォームを追加出来ます</span> <img src="./cmn/arrow-gfno.png" alt="追加" class="icon2"></small>\n|;
		} elsif ($ref_no == 5) { 
			$att_msg .= qq|<small><span style="color:#d83473">参照フォームは５個が限度です</span> <img src="./cmn/arrow-gfno01.png" alt="追加" class="icon2"></small>\n|;
		} else { 
			$att_msg .= qq|<small><span style="color:#00885a">参照フォームの数を変更できます</span> <img src="./cmn/arrow-gfno.png" alt="追加" class="icon2"></small>\n|;
		}

	# プルダウン作成（メール表示）
	my $op_smail = make_ops($cook[3]);

	# 画像認証作成
	my ($str_plain,$str_crypt,$regikey,$JavaScript);
	if ($cf{use_captcha} > 0) {
		require $cf{captcha_pl};
		($str_plain, $str_crypt, $regikey, $JavaScript) = cap::make( $cf{captcha_key}, $cf{cap_len} );
		$JavaScript=&mkJavaScript($regikey,'myFORM') if($cf{cap_len} == 3);
	}

	open(IN,"$cf{tmpldir}/form.html") or &error("open err: form.html");
	my $tmpl = join('', <IN>);
	close(IN);

	# 画像投稿
	if (!$cf{image_upl}) {
		$tmpl =~ s/<!-- image_begin -->.+<!-- image_end -->//s;
	}

	# 文字置き換え
	$tmpl =~ s/!([a-z]+_cgi)!/$cf{$1}/g;
	$tmpl =~ s/!post_cgi!/$cf{post_cgi}/g;
	$tmpl =~ s/!bbs_title!/$cf{bbs_title}/g;
	$tmpl =~ s|!bbs_css!|$cf{cmnurl}/wforum.css|g;
	$tmpl =~ s|!bbs_js!|$cf{cmnurl}/bbs.js|g;
	$tmpl =~ s/!str_crypt!/$str_crypt/g;
	$tmpl =~ s/!name!/$cook[0]/;
	$tmpl =~ s/!email!/$cook[1]/;
        $tmpl =~ s/<!-- op_smail -->/$op_smail/;
	$tmpl =~ s/!url!/$cook[2]/;
	$tmpl =~ s|!ico:(\w+\.\w+)!|<img src="$cf{icourl}/$1">|g;
	$tmpl =~ s/<!-- attach -->/$attach/g;
	$tmpl =~ s/<!-- add_attach -->/$op_attach/g;
	$tmpl =~ s/!att_msg!/$att_msg/g;

		$ref_no =~ s/0/０/g;
		$ref_no =~ s/1/１/g;
		$ref_no =~ s/2/２/g;
		$ref_no =~ s/3/３/g;
		$ref_no =~ s/4/４/g;
		$ref_no =~ s/5/５/g;

	$tmpl =~ s/!ref_no!/$ref_no/g;

	# 表示
	print "Content-type: text/html; charset=utf-8\n\n";
	footer($tmpl);
}

#-----------------------------------------------------------
#  アルバム機能
#-----------------------------------------------------------
sub view_page {
	# ページ数定義
	my $pg = $in{pg} || 0;

	# 画像サイズ再定義
	$cf{max_img_w} = $cf{alb_img_w};
	$cf{max_img_h} = $cf{alb_img_h};

	# テンプレート認識
	open(IN,"$cf{tmpldir}/view.html") or error("open err: view.html");
	my $tmpl = join('', <IN>);
	close(IN);

	$tmpl =~ s|!bbs_css!|$cf{cmnurl}/wforum.css|g;
	$tmpl =~ s|!bbs_js!|$cf{cmnurl}/bbs.js|g;

	# テンプレート分割
	my ($head,$loop,$foot) = split(/<!-- loop -->/,$tmpl);

	# データ読み込み
	my ($i,@img);
	open(IN,"$cf{logfile}") or error("open err: $cf{logfile}");
	while (<IN>) {
		my ($no,$re,$lx,$sub,$eml,$url,$nam,$dat,$msg,$lt,$ho,$pw,$wrap,$oya,$sml,$res,$e1,$w1,$h1,$myid,$key,$e2,$w2,$h2,$e3,$w3,$h3,$e4,$w4,$h4,$e5,$w5,$h5,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5) = split(/<>/);
		next if ($e1 !~ /(jpg|png|gif)$/);

		$i++;
		next if ($i < $pg + 1);
		next if ($i > $pg + $cf{max_albums});

		# 画像データ収集
		push(@img,"$no\t$sub\t$nam\t$e1\t$w1\t$h1\t$e2\t$w2\t$h2\t$e3\t$w3\t$h3\t$e4\t$w4\t$h4\t$e5\t$w5\t$h5\t$imgtxt1\t$imgtxt2\t$imgtxt3\t$imgtxt4\t$imgtxt5");
	}
	close(IN);

	# 繰越ボタン
	my $page_btn = view_pgbtn($i,$pg,'&mode=view',$cf{max_albums});

	# 文字置換
	for ($head, $foot) {
		s/!([a-z]+_cgi)!/$cf{$1}/g;
		s/!page_btn!/$page_btn/g;
		s/!bbs_title!/$cf{bbs_title}/g;
	}

	# 画面展開
	print "Content-type: text/html; charset=utf-8\n\n";
	print $head;

	foreach (@img) {
		my ($no,$sub,$nam,$e1,$w1,$h1,$e2,$w2,$h2,$e3,$w3,$h3,$e4,$w4,$h4,$e5,$w5,$h5,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5) = split(/\t/);

		my $clip;
			$clip = image($no,$e1,$w1,$h1,$e2,$w2,$h2,$e3,$w3,$h3,$e4,$w4,$h4,$e5,$w5,$h5,$sub,$nam,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5);
			$clip &&= "<p>$clip</p>";

		my $tmp = $loop;
		$tmp =~ s/!num!/$no/g;
		$tmp =~ s/!image!/$clip/g;
		$tmp =~ s/!caption!/$sub/g;
		print $tmp;
	}

	# フッタ
	footer($foot);
}

#-----------------------------------------------------------
#  繰越ボタン作成 (アルバム)
#-----------------------------------------------------------
sub view_pgbtn {
	my ($i,$pg,$stat,$max) = @_;
	$max ||= 5;

	# ページ繰越定義
	my $next = $pg + $max;
	my $back = $pg - $max;

	# ページ繰越ボタン作成
	my @pg;
	if ($back >= 0 || $next < $i) {
		my $flg;
		my ($w,$x,$y,$z) = (0,1,0,$i);
		while ($z > 0) {
			if ($pg == $y) {
				$flg++;
				push(@pg,qq!<li><span>$x</span></li>!);
			} else {
				push(@pg,qq!<li><a href="$cf{bbs_cgi}?pg=$y$stat">$x</a></li>!);
			}
			$x++;
			$y += $max;
			$z -= $max;

			if ($flg) { $w++; }
			last if ($w >= 5 && @pg >= 10);
		}
	}
	while( @pg >= 11 ) { shift(@pg); }
	my $ret = join('', @pg);
	if ($back >= 0) {
		$ret = qq!<li><a href="$cf{bbs_cgi}?pg=$back$stat">&laquo;</a></li>\n! . $ret;
	}
	if ($next < $i) {
		$ret .= qq!<li><a href="$cf{bbs_cgi}?pg=$next$stat">&raquo;</a></li>\n!;
	}
	# 結果を返す
	return $ret ? qq|<ul class="pager">$ret</ul>| : '';
}

#-----------------------------------------------------------
#  画像表示 (アルバム画像)
#-----------------------------------------------------------
sub image {
	my $no = shift;
	my (%e,%w,%h,$sub,$nam,%imgtxt);
	($e{1},$w{1},$h{1},$e{2},$w{2},$h{2},$e{3},$w{3},$h{3},$e{4},$w{4},$h{4},$e{5},$w{5},$h{5},$sub,$nam,$imgtxt{1},$imgtxt{2},$imgtxt{3},$imgtxt{4},$imgtxt{5}) = @_;

	my $ret;
	foreach my $i (1 .. 5) {
		next if (!$e{$i});

		if ($e{$i} =~ /^\.(jpg|gif|png)$/) {
			if (-f "$cf{upldir}/$no-s-$i$e{$i}") {
				$ret .= qq|<div class="photo"><a href="$cf{uplurl}/$no-$i$e{$i}" class="lightview" data-lightview-title="投稿者：$nam　 記事タイトル：$sub ファイル名：$imgtxt{$i}" data-lightview-group="light-set" data-lightview-group-options="skin: 'mac'"><img src="$cf{uplurl}/$no-s-$i$e{$i}" class="alb_snapshot"></a><br />No. $no<br />投稿者 : $nam<br />タイトル : $sub<br />ファイル名：$imgtxt{$i}</div>|;

			} else {
				my ($w,$h) = resize($w{$i},$h{$i});
				$ret .= qq|<div class="photo"><a href="$cf{uplurl}/$no-$i$e{$i}" class="lightview" data-lightview-title="投稿者：$nam　 記事タイトル：$sub ファイル名：$imgtxt{$i}" data-lightview-group="light-set" data-lightview-group-options="skin: 'mac'"><img src="$cf{uplurl}/$no-$i$e{$i}" width="$w" height="$h" class="alb_snapshot"></a><br />No. $no<br />投稿者 : $nam<br />タイトル : $sub<br />ファイル名：$imgtxt{$i}</div>|;
			}
		}
	}
	return $ret;
}

#-----------------------------------------------------------
#  留意事項
#-----------------------------------------------------------
sub note_page {
	open(IN,"$cf{tmpldir}/note.html") or &error("open err: note.html");
	my $tmpl = join('', <IN>);
	close(IN);

	$tmpl =~ s/!([a-z]+_cgi)!/$cf{$1}/g;
	$tmpl =~ s|!bbs_css!|$cf{cmnurl}/wforum.css|g;
	$tmpl =~ s|!bbs_js!|$cf{cmnurl}/bbs.js|g;
	$tmpl =~ s/!treehead!/$cf{treehead_top}/g;
	$tmpl =~ s/!bbs_title!/$cf{bbs_title}/g;
	$tmpl =~ s|!ico:(\w+\.\w+)!|<img src="$cf{icourl}/$1">|g;

	print "Content-type: text/html; charset=utf-8\n\n";
	print $tmpl;

	# フッタ
	footer();

	exit;
}

#-----------------------------------------------------------
#  検索フォーム
#-----------------------------------------------------------
sub find_form {
	my ($i,$pgbtn,@data) = search($cf{logfile},'cur') if ($in{q} ne '');

	# プルダウン作成（検索条件）
	my %op = make_op();

	# テンプレート認識
	open(IN,"$cf{tmpldir}/find.html") or &error("open err: find.html");
	my $tmpl = join('', <IN>);
	close(IN);

	$tmpl =~ s/!(bbs_title|cmnurl|bbs_cgi)!/$cf{$1}/g;
	$tmpl =~ s/!q!/$in{q}/g;
	$tmpl =~ s|!bbs_css!|$cf{cmnurl}/wforum.css|g;
	$tmpl =~ s|!bbs_js!|$cf{cmnurl}/bbs.js|g;

	# テンプレート分割
	my ($head,$loop,$foot) = $tmpl =~ /(.+)<!-- msg_begin -->(.+)<!-- msg_end -->(.+)/s
			? ($1,$2,$3) : &error("テンプレートが不正です");

	# 文字置き換え
	foreach ($head,$foot) {
		s/!bbs_cgi!/$cf{bbs_cgi}/g;
		s/!bbs_title!/$cf{bbs_title}/g;
		s/!word!/$in{q}/g;
		s/<!-- op_(\w+) -->/$op{$1}/g;

		# ワード検索
		if ($in{q}) {
			s/!result!/$i/g;
			s/!page_btn!/$pgbtn/g;
		} else {
			s|<!-- find -->.+?<!-- /find -->||sg;
		}
	}

	# 画面展開
	print "Content-type: text/html; charset=utf-8\n\n";
	print $head;

	foreach my $data (@data) {
		my ($no,$reno,$lx,$sub,$eml,$url,$name,$date,$msg,$t,$ho,$pw,$wrap,$oya,$sml,$res,$e1,$w1,$h1,$myid,$key,$e2,$w2,$h2,$e3,$w3,$h3,$e4,$w4,$h4,$e5,$w5,$h5,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5) = split(/<>/, $data);
		$msg = autolink($msg) if ($cf{autolink});

		# 返信フォーム
		my $resfm = 1;
		if ($key != 1) {
			$resfm = 0;
		}

		# 引用部色変更
		if ($cf{ref_col}) {
			$msg =~ s/([\>]|^)(&gt;[^<]*)/$1<font color="$cf{ref_col}">$2<\/font>/g;
			$msg =~ s/([\>]|^)(&amp;gt;[^<]*)/$1<font color="$cf{ref_col}">$2<\/font>/g;
		}

		$msg = tag($msg);

		# 画像付加
		$msg = put_find1($no,$msg,$e1,$w1,$h1,$imgtxt1) if ($e1);
		$msg = put_find2($no,$msg,$e2,$w2,$h2,$imgtxt2) if ($e2);
		$msg = put_find3($no,$msg,$e3,$w3,$h3,$imgtxt3) if ($e3);
		$msg = put_find4($no,$msg,$e4,$w4,$h4,$imgtxt4) if ($e4);
		$msg = put_find5($no,$msg,$e5,$w5,$h5,$imgtxt5) if ($e5);

		$date = get_time($t);

		# e-mail
		if ($eml && $sml eq '0') { $name = qq|<a href="mailto:$eml">$name</a>|; }
		# URL
		$url &&= qq|<a href="$url" target="_blank"><img src="$cf{ico_home}" class="icon"></a>|;

		my $tmp = $loop;
		$tmp =~ s/!sub!/$sub/g;
		$tmp =~ s/!num!/$no/g;
		$tmp =~ s/!date!/$date/g;
		$tmp =~ s/!name!/$name/g;
		$tmp =~ s/!url!/$url/g;
		$tmp =~ s/!msg!/$msg/g;
		$tmp =~ s/!bbs_cgi!/$cf{bbs_cgi}/g;
		$tmp =~ s/!reno!/$reno/g;
		$tmp =~ s/!oya!/$oya/g;

	# ロックコメント
	if ($key == 0) {
		$tmp =~ s|<!-- lock_com -->|<div align="right"><div id="lock"><font color="#8080c0">このスレッドは管理者によって</font><font color="red"><b>ロック</b></font><font color="#8080c0">されていますので、返信並びに編集は出来ません！</font></div></div>|g;
	} elsif ($key == 2)  {
		$tmp =~ s|<!-- solved_com -->|<div align="right"><div id="solved"><font color="#8080c0">このスレッドは</font><font color="red"><b>解決済</b></font><font color="#8080c0">ですので、返信並びに編集は出来ません！</font></div></div>|g;
	}

		# 返信ボタン
		if (!$resfm) {
			$tmp =~ s/<!-- edit_begin -->.+?<!-- edit_end -->//sg;
		}

		print $tmp;
	}

	# フッタ
	footer($foot);
}

#-----------------------------------------------------------
#  検索実行
#-----------------------------------------------------------
sub search {
	my ($file,$stat) = @_;

	$in{q} =~ s/　/ /g;
	my @q = split(/\s+/, $in{q});

	# UTF-8定義
	my $byte1 = '[\x00-\x7f]';
	my $byte2 = '[\xC0-\xDF][\x80-\xBF]';
	my $byte3 = '[\xE0-\xEF][\x80-\xBF]{2}';
	my $byte4 = '[\xF0-\xF7][\x80-\xBF]{3}';

	# 表示数
	$in{view} =~ s/\D//g;
	$in{view} ||= 10;

	# ページ数
	$in{page} =~ s/\D//g;
	$in{page} ||= 0;

	# ファイル展開
	my $i = 0;
	my @data;
	open(IN,"$file") or &error("open err: $file");
	my $top = <IN> if ($stat eq 'cur');
	while (<IN>) {
		chomp;
		my ($no,$reno,$lx,$sub,$eml,$url,$name,$date,$msg,$t,$ho,$pw,$wrap,$oya,$sml,$res,$e1,$w1,$h1,$myid,$key,$e2,$w2,$h2,$e3,$w3,$h3,$e4,$w4,$h4,$e5,$w5,$h5,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5) = split(/<>/);
		next if ($pw eq 'DEL');

		# 検索
		my $flg;
		foreach my $q (@q) {
			if ("$sub $url $name $msg $imgtxt1 $imgtxt2 $imgtxt3 $imgtxt4 $imgtxt5" =~ /^(?:$byte1|$byte2|$byte3|$byte4)*?\Q$q\E/i) {
				$flg++;
				if (!$in{cond}) { last; } # OR
			} else {
				if ($in{cond}) { $flg = 0; last; } # AND
			}
		}
		# 一致の場合
		if ($flg) {
			$i++;
			next if ($i < $in{page} + 1);
			next if ($i > $in{page} + $in{view});

			push(@data,$_);
		}
	}
	close(IN);

	# ページ移動ボタン
	my $pgbtn = find_pgbtn($i,$stat);

	return ($i,$pgbtn,@data);
}

#-----------------------------------------------------------
#  検索用ページ移動ボタン
#-----------------------------------------------------------
sub find_pgbtn {
	my ($i, $stat) = @_;

	# 繰り越しページ定義
	my $next = $in{page} + $in{view};
	my $back = $in{page} - $in{view};

	# キーワードエンコード化
	my $enwd = url_enc($in{q});

	# パラメータ
	my $param;
	if ($stat eq 'cur') {
		$param = $mode;
	} else {
		$param = "$mode&amp;pno=$in{pno}&amp;find=1";
	}

	# ページ移動ボタン作成
	my @pg;
	if ($back >= 0 || $next < $i) {
		my $flg;
		my ($w,$x,$y,$z) = (0,1,0,$i);
		while ($z > 0) {
			if ($in{page} == $y) {
				$flg++;
				push(@pg,qq!<li><span>$x</span></li>!);
			} else {
				push(@pg,qq!<li><a href="$cf{bbs_cgi}?mode=$param&page=$y&q=$enwd&view=$in{view}&cond=$in{cond}">$x</a></li>!);
		}
		$x++;
		$y += $in{view};
		$z -= $in{view};

			if ($flg) { $w++; }
			last if ($w >= 5 && @pg >= 10);
		}
	}
	while( @pg >= 11 ) { shift(@pg); }
	my $ret = join('', @pg);
	if ($back >= 0) {
		$ret = qq!<li><a href="$cf{bbs_cgi}?mode=$param&page=$back&q=$enwd&view=$in{view}&cond=$in{cond}">&laquo;</a></li>\n! . $ret;
	}
	if ($next < $i) {
		$ret .= qq!<li><a href="$cf{bbs_cgi}?mode=$param&page=$next&q=$enwd&view=$in{view}&cond=$in{cond}">&raquo;</a></li>\n!;
	}
	
	# 結果を返す
	return $ret ? qq|<ul class="pager">$ret</ul>| : '';
}

#-----------------------------------------------------------
#  新着用ページ移動ボタン
#-----------------------------------------------------------
sub new_pgbtn {
	my $i = shift;

	my $next = $in{page} + $cf{new_pgbtn};
	my $back = $in{page} - $cf{new_pgbtn};

	# 表示範囲
	my $from = $in{page} + 1;
	my $to = $in{page} + $cf{new_pgbtn} > $i ? $i : $in{page} + $cf{new_pgbtn};
	my $pg_list = "$from - $to [ $i件中 ]";

	# ページ移動ボタン表示
	my @pg;
	if ($back >= 0 || $next < $i) {
		my $flg;
		my ($w,$x,$y,$z) = (0,1,0,$i);
		while ($z > 0) {
		if ($in{page} == $y) {
			$flg++;
			push(@pg,qq!<li><span>$x</span></li>!);
		} else {
			$z++;
			push(@pg,qq!<li><a href="$cf{bbs_cgi}?mode=relate&job=new&page=$y">$x</a></li>!);
		}
		$x++;
		$y += $cf{new_pgbtn};
		$z -= $cf{new_pgbtn};

			if ($flg) { $w++; }
			last if ($w >= 5 && @pg >= 10);
		}
	}
	while( @pg >= 11 ) { shift(@pg); }
	my $ret = join('', @pg);
	if ($back >= 0) {
		$ret = qq!<li><a href="$cf{bbs_cgi}?page=$back">&laquo;</a></li>\n! . $ret;
	}
	if ($next < $i) {
		$ret .= qq!<li><a href="$cf{bbs_cgi}?page=$next">&raquo;</a></li>\n!;
	}
	
	# 結果を返す
	return $ret ? qq|<ul class="pager">$pg_list&nbsp;&nbsp; $ret</ul>| : '';
}

#-----------------------------------------------------------
#  パスワードフォーム
#-----------------------------------------------------------
sub pwd_form {

	$in{no} =~ s/\D//g;

	my ($flg,@new,%r,%v,$pwd,$myid);
	open(IN,"$cf{logfile}") or &error("open err: $cf{logfile}");
	my $top = <IN>;
	while (<IN>) {
		chomp;
		my ($no,$reno,$lx,$sub,$eml,$url,$nam,$dat,$msg,$lt,$ho,$pw,$wrap,$oya,$sml,$res,$e1,$w1,$h1,$myid,$key,$e2,$w2,$h2,$e3,$w3,$h3,$e4,$w4,$h4,$e5,$w5,$h5,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5) = split(/<>/);

		if ($in{oya} == $oya) { push(@new,$_); }
		elsif ($flg && $in{oya} != $oya) { last; }

		if ($in{no} == $no) {
			$flg = 1;
			$r{pwd} = $pw;
			$r{no}  = $no;
			$r{lx}  = $lx;
			$r{dat}  = $dat;
			$r{name}  = $nam;
			$r{email} = $eml;
			$r{url} = $url;
			$r{msg} = $msg;
			$r{tim} = $lt;
			$r{sub} = $sub;
			$r{myid} = $myid;
			$r{oya} = $oya;
			$r{sml} = $sml;
			$r{e1} = $e1;
			$r{w1} = $w1;
			$r{h1} = $h1;
			$r{e2} = $e2;
			$r{w2} = $w2;
			$r{h2} = $h2;
			$r{e3} = $e3;
			$r{w3} = $w3;
			$r{h3} = $h3;
			$r{e4} = $e4;
			$r{w4} = $w4;
			$r{h4} = $h4;
			$r{e5} = $e5;
			$r{w5} = $w5;
			$r{h5} = $h5;
			$r{imgt1} = $imgtxt1;
			$r{imgt2} = $imgtxt2;
			$r{imgt3} = $imgtxt3;
			$r{imgt4} = $imgtxt4;
			$r{imgt5} = $imgtxt5;
		}
	}
	close(IN);

	# パスワード未設定
	error("この記事はパスワード未設定のため編集できません") if ($r{pwd} eq '');

	# レスメッセージ
	my %c;
	$c{message} = "\n&gt; $r{msg}";
	$c{message} =~ s|<br( /)?>|\n&gt; |ig;

	# レスタイトル
	if ($r{sub} =~ /^Re\^(\d+)\:(.*)/) {
		my $renum = $1 + 1;
		$c{sub} = "Re\^$renum:$2";
	}
	elsif ($r{sub} =~ /^Re\:(.*)/) { $c{sub} = "Re\^2:$1"; }
	else { $c{sub} = "Re: $r{sub}"; }

	# 投稿日時
	$r{date} = get_time($r{tim});

	# 処理選択オプション
	my $op_job = qq|<input type="radio" name="mode" value="user_edit" checked>記事を修正&nbsp;&nbsp;|;
	if ($r{lx} == 0) {
		$op_job .= qq|<input type="radio" name="mode" value="user_lock">スレッド解決済&nbsp;&nbsp;|;
		$op_job .= qq|<input type="radio" name="mode" value="user_dele">記事を削除|;
	} else {
		$op_job .= qq|<input type="radio" name="mode" value="user_dele">記事を削除|;
	}

	my ($no,$reno,$lx,$sub,$eml,$url,$nam,$dat,$msg,$lt,$ho,$pw,$wrap,$oya,$sml,$res,$e1,$w1,$h1,$myid,$key,$e2,$w2,$h2,$e3,$w3,$h3,$e4,$w4,$h4,$e5,$w5,$h5,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5) = split(/<>/);
	open(IN,"$cf{tmpldir}/pwd.html") or &error("open err: pwd.html");
	my $tmpl = join('', <IN>);
	close(IN);

	# ページ数
	$in{page} =~ s/\D//g;
	$in{page} ||= 0;

	my $i = 0;
	if ($reno == 0) { $i++; }

	# 画像投稿用
	if (!$cf{image_upl}) {$tmpl =~ s/<!-- image_begin -->.+<!-- image_end -->//s;}

	# 文字置換
	$tmpl =~ s/!r_(\w+)!/$r{$1}/g;
	$tmpl =~ s/!([a-z]+_cgi)!/$cf{$1}/g;
	$tmpl =~ s/!post_cgi!/$cf{post_cgi}/g;
	$tmpl =~ s|!bbs_css!|$cf{cmnurl}/wforum.css|g;
	$tmpl =~ s|!bbs_js!|$cf{cmnurl}/bbs.js|g;
	$tmpl =~ s|!ico:(\w+\.\w+)!|<img src="$cf{icourl}/$1" class="icon">|g;
	$tmpl =~ s/!bbs_title!/$cf{bbs_title}/g;
	$tmpl =~ s/!no!/$in{no}/g;
	$tmpl =~ s/!reno!/$reno/g;
	$tmpl =~ s/!page!/$in{page}/g;
	$tmpl =~ s/!date!/$r{date}/g;
	$tmpl =~ s/<!-- op_job -->/$op_job/g;

	# 画像用
	my ($image01,$image02,$image03,$image04,$image05);
	if ($r{e1}) {
		 $image01 .= qq|&nbsp; [<a href="$cf{uplurl}/$r{no}-1$r{e1}" target="_blank">閲覧/開く</a>]\n|;
	} else { $image01 .= qq|&nbsp; (<span style="color:#941f57; font-size:9pt"> not attach </span>)\n|;}
		 $tmpl =~ s/<!-- image01 -->/$image01/g;

	if ($r{e2}) {
		 $image02 .= qq|&nbsp; [<a href="$cf{uplurl}/$r{no}-2$r{e2}" target="_blank">閲覧/開く</a>]\n|;
	} else { $image02 .= qq|&nbsp; (<span style="color:#941f57; font-size:9pt"> not attach </span>)\n|;}
		 $tmpl =~ s/<!-- image02 -->/$image02/g;

	if ($r{e3}) {
		 $image03 .= qq|&nbsp; [<a href="$cf{uplurl}/$r{no}-3$r{e3}" target="_blank">閲覧/開く</a>]\n|;
	} else { $image03 .= qq|&nbsp; (<span style="color:#941f57; font-size:9pt"> not attach </span>)\n|;}
		 $tmpl =~ s/<!-- image03 -->/$image03/g;

	if ($r{e4}) {
		 $image04 .= qq|&nbsp; [<a href="$cf{uplurl}/$r{no}-4$r{e4}" target="_blank">閲覧/開く</a>]\n|;
	} else { $image04 .= qq|&nbsp; (<span style="color:#941f57; font-size:9pt"> not attach </span>)\n|;}
		 $tmpl =~ s/<!-- image04 -->/$image04/g;

	if ($r{e5}) {
		 $image05 .= qq|&nbsp; [<a href="$cf{uplurl}/$r{no}-5$r{e5}" target="_blank">閲覧/開く</a>]\n|;
	} else { $image05 .= qq|&nbsp; (<span style="color:#941f57; font-size:9pt"> not attach </span>)\n|;}
		 $tmpl =~ s/<!-- image05 -->/$image05/g;

	print "Content-type: text/html; charset=utf-8\n\n";
	print $tmpl;

	# フッタ
	footer();
}

#-----------------------------------------------------------
#  過去ログ画面
#-----------------------------------------------------------
sub past_page {
	# 過去ログ番号
	open(IN,"$cf{nofile}") or &error("open err: $cf{nofile}");
	my $pastnum = <IN>;
	close(IN);

	my $pastnum = sprintf("%04d", $pastnum);
	$in{pno} =~ s/\D//g;
	$in{pno} ||= $pastnum;

	# プルダウンタグ作成
	my $op_pno;
	for ( my $i = $pastnum; $i > 0; $i-- ) {
		$i = sprintf("%04d", $i);

		if ($in{pno} == $i) {
			$op_pno .= qq|<option value="$i" selected>$i\n|;
		} else {
			$op_pno .= qq|<option value="$i">$i\n|;
		}
	}

	# 初期化
	my ($list,$i,$pgbtn,$find,@tree);

	# 記事閲覧
	if ($in{job} eq 'read') {

		@tree = past_read();

	# 関連ツリー作成
	my $x = 0;
	my $x = 0;
	my ($pad,$j,@x);
	foreach (@tree) {
		my ($no,$re,$lx,$sub,$eml,$url,$nam,$dat,$msg,$lt,$ho,$pw,$wrap,$oya,$sml,$res,$e1,$w1,$h1,$myid,$key,$e2,$w2,$h2,$e3,$w3,$h3,$e4,$w4,$h4,$e5,$w5,$h5,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5) = split(/<>/);

	# アイコン判断
	if ($lx != 0) {
		my $ico;
		if ($key eq '0') { $ico = 'res_lock'; }
		elsif ($key eq '2') { $ico = 'res_lock_user'; }
		elsif ($e1) { $ico = 'res_image'; }
		else { $ico = 'res_normal'; }
		$cf{res_treehead} =  qq|<img src="$cf{icourl}/$cf{res_icon}->{$ico}" border="0" style="vertical-align:middle;">|;
	} else {
		my $ico;
		if ($key eq '0') { $ico = 0; }
		elsif ($key eq '2') { $ico = 2; }
		elsif ($e1) { $ico = 'image'; }
		else { $ico = 1; }
		$cf{treehead} =  qq|<img src="$cf{icourl}/$cf{fld_icon}->{$ico}" border="0" style="vertical-align:middle;" title="一括\表\示" alt="一括\表\示">|;
	}

		$x[$lx+1]=$res;
		$x[$lx]--;
		$pad = "";
		for ($j=0; $j<$lx; $j++){
		$pad .= ($x[$j]<=0)?"　 ":"<img src=\"$cf{tree_ico}/tree_i.gif\" align=\"top\" width=\"19\" height=\"18\" border=\"0\">";
		}
		$pad .= ($x[$lx]<=0)?"<img src=\"$cf{tree_ico}/tree_f.gif\" align=\"top\" width=\"19\" height=\"18\" border=\"0\">":"<img src=\"$cf{tree_ico}/tree_e.gif\" align=\"top\" width=\"19\" height=\"18\" border=\"0\">";

		if ($lx != 0) {
			$list .= qq|$pad|;
			$list .= qq|$cf{res_treehead} <a href="$cf{bbs_cgi}?no=$no&reno=$re&oya=$oya&mode=read&page=$in{page}">$sub</a> - <span class="tree-date">$dat</span> |;
		} else {
			$list .= qq|<img src="$cf{tree_ico}/tree_b.gif" align="top" width="15" height="17"><a href="$cf{bbs_cgi}?mode=relate&no=$no&page=$in{page}">$cf{treehead}</a> <a href="$cf{bbs_cgi}?no=$no&reno=$re&oya=$oya&mode=read&page=$in{page}">$sub</a> - <span class="tree-date"><B>$nam</B> $dat</span>|;
		}

		if ($in{no} == $no) {
			$list .= qq| <b style="color:$cf{num_em_col}">No.$no</b><br>\n|;
		} else {
			$list .= qq| <span style="color:$cf{num_st_col}">No.$no</span><br>\n|;
		}
	}
	$list .= qq|</td></tr></table>\n|;

	# ワード検索
	} elsif ($in{find} && $in{word} ne '') {

		# 対象ログ定義
		my $file = "$cf{pastdir}/" . sprintf("%04d", $in{pno}) . ".cgi";

		# 検索
		($i,$pgbtn,@tree) = search($file, 'past');

		# 結果
		$list = "<blockquote>検索結果：<b>$i</b>件 &nbsp;&nbsp; $pgbtn</blockquote>\n";

	# 過去ログTOPリスト
	} else {

		# ツリー展開
		($list,$i) = &open_tree("$cf{pastdir}/$in{pno}.cgi", 'past');

		# ページ移動ボタン
		$pgbtn = page_btn($i,'past');
	}

	# プルダウン作成（検索条件）
	my %op = make_op();

	# テンプレート認識
	open(IN,"$cf{tmpldir}/past.html") or &error("open err: past.html");
	my $tmpl = join('', <IN>);
	close(IN);

	# 検索
	if ($find) {
		$tmpl =~ s/!result!/$i/;
	} else {
		$tmpl =~ s|<!-- find -->.+?<!-- /find -->||s;
	}

	# 文字置き換え
	$tmpl =~ s|!bbs_css!|$cf{cmnurl}/wforum.css|g;
	$tmpl =~ s|!bbs_js!|$cf{cmnurl}/bbs.js|g;
	$tmpl =~ s|!ico:(\w+\.\w+)!|<img src="$cf{icourl}/$1">|g;
	$tmpl =~ s/!past_num!/$in{pno}/g;
	$tmpl =~ s/<!-- op_pno -->/$op_pno/g;
	$tmpl =~ s/!(bbs_cgi|cmnurl|bbs_title)!/$cf{$1}/g;
	$tmpl =~ s/<!-- list_tree -->/$list/g;
	$tmpl =~ s/<!-- page_btn -->/$pgbtn/g;
	$tmpl =~ s/!q!/$in{q}/g;
	$tmpl =~ s/<!-- op_(\w+) -->/$op{$1}/g;

	if ($in{job} eq 'read') {
		$tmpl =~ s|<!-- past_read -->|<button type="button" onclick="javascript:window.location='$cf{bbs_cgi}?mode=past'" class="color dark_sepia large button">過去ログTOP</button>|g;
	} else {
		$tmpl =~ s|<!-- past_cgi -->|<button type="button" onclick="javascript:window.location='$cf{bbs_cgi}'" class="color dark_sepia large button">TOP</button>|s;
	}

	# テンプレート分割
	my ($head,$loop,$foot) = $tmpl =~ m|(.+)<!-- article -->(.+?)<!-- /article -->(.+)|s
			? ($1,$2,$3)
			: error("テンプレート不正");

	# 画面展開
	print "Content-type: text/html; charset=utf-8\n\n";
	print $head;

	my %p;
	foreach my $tree (@tree) {
		($p{num},$p{sub},$p{email},$p{url},$p{name},$p{date},$p{msg},$p{sml}) = (split(/<>/, $tree))[0,3..8,14];
		$p{msg} = &autolink($p{msg}) if ($cf{autolink});

		# 引用色
		if ($cf{ref_col}) {
			$p{msg} =~ s/([\>]|^)(&gt;[^<]*)/$1<font color="$cf{ref_col}">$2<\/font>/g;
		}

		# e-mail
		if ($p{email} && $p{sml} eq '0') {
			$p{email} = qq|&lt;<a href="mailto:$p{email}" class="num">$p{email}</a>&gt;|;
		} else {
			$p{email} = '';
		}
		# URL
		if ($p{url}) {
			$p{url} = qq|<a href="$p{url}" class="num">$p{url}</a>|;
		}

		my $tmp = $loop;
		$tmp =~ s/!(\w+)!/$p{$1}/g;
		print $tmp;
	}

	# フッタ
	footer($foot);
}

#-----------------------------------------------------------
#  認証モード
#-----------------------------------------------------------
sub authent {
	# セッションモジュール取り込み
	require $cf{session_pl};

	# ログイン
	if ($in{mode} eq 'login') {
		# 入力チェック
		if ($in{id} eq '' || $in{pw} eq '') {
			error("IDまたはパスワードが未入力です");
		}
		# セッション作成
		make_ses($in{id},$in{pw},$cf{memfile},$cf{sesdir},$cf{authtime});

	# ログオフ
	} elsif ($in{mode} eq 'logoff') {
		# セッション削除
		del_ses($cf{sesdir});

		# 入室画面
		enter_form('cook_del');

	# 入室画面
	} elsif ($in{mode} eq 'enter') {
		enter_form();

	# セッション管理
	} else {
		session($cf{sesdir},$cf{bbs_cgi});
	}
}

#-----------------------------------------------------------
#  入室画面
#-----------------------------------------------------------
sub enter_form {
	my $ck = shift;

	# テンプレート読み込み
	open(IN,"$cf{tmpldir}/enter.html") or &error("open err: enter.html");
	my $tmpl = join('', <IN>);
	close(IN);

	# 文字置換
	$tmpl =~ s|!bbs_css!|$cf{cmnurl}/wforum.css|g;
	$tmpl =~ s|!bbs_js!|$cf{cmnurl}/bbs.js|g;
	$tmpl =~ s/!bbs_cgi!/$cf{bbs_cgi}/g;
	$tmpl =~ s/!bbs_title!/$cf{bbs_title}/g;

	# クッキー排除
	if ($ck eq 'cook_del') {
		set_cookie('CGISESSID','','del');
	}

	# 画面表示
	print "Content-type: text/html; charset=utf-8\n\n";
	print $tmpl;

	# フッタ
	footer();
}

#-----------------------------------------------------------
#  crypt照合
#-----------------------------------------------------------
sub decrypt {
	my ($in, $dec) = @_;

	my $salt = $dec =~ /^\$1\$(.*)\$/ ? $1 : substr($dec, 0, 2);
	if (crypt($in, $salt) eq $dec || crypt($in, '$1$' . $salt) eq $dec) {
		return 1;
	} else {
		return 0;
	}
}

#-----------------------------------------------------------
#  過去記事閲覧
#-----------------------------------------------------------
sub past_read {
	my ($flg,@tree);
	open(IN,"$cf{pastdir}/$in{pno}.cgi");
	while(<IN>) {
		my ($no,$reno,$lx,$sub,$eml,$url,$name,$date,$msg,$t,$ho,$pw,$wrap,$oya,$sml,$myid,$key) = split(/<>/);

		if ($in{oya} == $oya) { push(@tree,$_); }
		elsif ($flg && $in{oya} != $oya) { last; }
	}
	close(IN);

	return @tree;
}

#-----------------------------------------------------------
#  ページ移動ボタン
#-----------------------------------------------------------
sub page_btn {
	my ($i,$stat) = @_;

	# 繰り越しページ
	my $next = $in{page} + $cf{p_tree};
	my $back = $in{page} - $cf{p_tree};

	# 表示範囲
	my $from = $in{page} + 1;
	my $to = $in{page} + $cf{p_tree} > $i ? $i : $in{page} + $cf{p_tree};
	my $pg_list = "$from - $to [ $i件中 ]";

	# ページ移動ボタン表示
	my @pg;
	if ($back >= 0 || $next < $i) {
		my $flg;
		my ($w,$x,$y,$z) = (0,1,0,$i);
		while ($z > 0) {
		if ($in{page} == $y) {
			$flg++;
			push(@pg,qq!<li><span>$x</span></li>!);
		} elsif ($stat eq 'past') {
			push(@pg,qq!<li><a href="$cf{bbs_cgi}?page=$y&mode=past&pno=$in{pno}">$x</a></li>!);
		} else {
			push(@pg,qq!<li><a href="$cf{bbs_cgi}?page=$y">$x</a></li>!);
		}
		$x++;
		$y += $cf{p_tree};
		$z -= $cf{p_tree};

		if ($flg) { $w++; }
		last if ($w >= 5 && @pg >= 10);
		}
	}
	while( @pg >= 11 ) { shift(@pg); }
	my $ret = join('', @pg);
	if ($back >= 0) {
		$ret = qq!<li><a href="$cf{bbs_cgi}?page=$back">&laquo; 前へ</a></li>\n! . $ret;
	}
	if ($next < $i) {
		$ret .= qq!<li><a href="$cf{bbs_cgi}?page=$next">次へ &raquo;</a></li>\n!;
	}
	
	# 結果を返す
	return $ret ? qq|<ul class="pager">$pg_list&nbsp;&nbsp; $ret</ul>| : '';
}

#-----------------------------------------------------------
#  カウンタ処理
#-----------------------------------------------------------
sub bbs_count {
	# IP取得
	my $addr = $ENV{REMOTE_ADDR};

	# 閲覧時のみカウントアップ
	my $cntup = $in{mode} eq '' ? 1 : 0;

	# カウントファイルを読みこみ
	open(LOG,"+< $cf{cntfile}") or error("open err: $cf{cntfile}");
	eval "flock(LOG, 2);";
	my $count = <LOG>;

	# IPチェックとログ破損チェック
	my ($cnt,$ip) = split(/:/,$count);
	if ($addr eq $ip or $cnt eq "") { $cntup = 0; }

	# カウントアップ
	if ($cntup) {
		$cnt++;
		seek(LOG, 0, 0);
		print LOG "$cnt:$addr";
		truncate(LOG, tell(LOG));
	}
	close(LOG);

	# 桁数調整
	while(length($cnt) < $cf{mini_fig}) { $cnt = '0' . $cnt; }
	my @cnts = split(//,$cnt);

	# GIFカウンタ表示
	my $counter;
	if ($cf{counter} == 2) {
		foreach (0 .. $#cnts) {
			$counter .= qq|<img src="$cf{gif_path}/$cnts[$_].gif\" alt="$cnts[$_]" />|;
		}

	# テキストカウンタ表示
	} else {
		$counter = qq|<span style="color:$cf{cntcol}">$cnt</span>\n|;
	}
	return $counter;
}

#-----------------------------------------------------------
#  URLエンコード
#-----------------------------------------------------------
sub url_enc {
	local($_) = @_;

	s/(\W)/'%' . unpack('H2', $1)/eg;
	s/\s/+/g;
	$_;
}

#-----------------------------------------------------------
#  プルダウン作成 [ 検索条件 ]
#-----------------------------------------------------------
sub make_op {
	my %op;
	my %cond = (1 => 'AND', 0 => 'OR');
	foreach (1,0) {
		if ($in{cond} eq $_) {
			$op{cond} .= qq|<option value="$_" selected>$cond{$_}\n|;
		} else {
			$op{cond} .= qq|<option value="$_">$cond{$_}\n|;
		}
	}
	for ( my $i = 10; $i <= 30; $i += 5 ) {
		if ($in{view} == $i) {
			$op{view} .= qq|<option value="$i" selected>$i件\n|;
		} else {
			$op{view} .= qq|<option value="$i">$i件\n|;
		}
	}
	return %op;
}

#-----------------------------------------------------------
#  プルダウン作成 [ メール表示 ]
#-----------------------------------------------------------
sub make_ops {
	my $sml = shift;

	my %ops = (0 => '表示', 1 => '非表示');
	my $op_sml;
	foreach (0,1) {
		if ($sml eq $_) {
			$op_sml .= qq|<option value="$_" selected>$ops{$_}\n|;
		} else {
			$op_sml .= qq|<option value="$_">$ops{$_}\n|;
		}
	}
	return $op_sml;
}

#-----------------------------------------------------------
#  自動リンク
#-----------------------------------------------------------
sub autolink {
	my $text = shift;

	$text =~ s/(s?https?:\/\/([\w-.!~*'();\/?:\@=+\$,%#]|&amp;)+)/<a href="$1" target="_blank">$1<\/a>/g;
	return $text;
}

#-----------------------------------------------------------
#  参照フォームクッキー取得(新規投稿)
#-----------------------------------------------------------
sub ref_get_cookie {

	# クッキー取得
	my $cook = $ENV{HTTP_COOKIE};
	
	# 該当IDを取り出す
	my %cook;
	foreach ( split(/;/,$cook) ) {
		my ($key,$val) = split(/=/);
		$key =~ s/\s//g;
		$cook{$key} = $val;
	}
	
	# URLデコード
	my @cook;
	foreach ( split(/<>/,$cook{$cf{cookie_id2}}) ) {
		s/%([0-9A-Fa-f][0-9A-Fa-f])/pack("H2", $1)/eg;
		s/[&"'<>]//g;
		
		push(@cook,$_);
	}
	return @cook;
}

#-----------------------------------------------------------
#  参照フォームクッキー取得(返信投稿)
#-----------------------------------------------------------
sub refres_get_cookie {

	# クッキー取得
	my $cook = $ENV{HTTP_COOKIE};
	
	# 該当IDを取り出す
	my %cook;
	foreach ( split(/;/,$cook) ) {
		my ($key,$val) = split(/=/);
		$key =~ s/\s//g;
		$cook{$key} = $val;
	}
	
	# URLデコード
	my @cook;
	foreach ( split(/<>/,$cook{$cf{cookie_id4}}) ) {
		s/%([0-9A-Fa-f][0-9A-Fa-f])/pack("H2", $1)/eg;
		s/[&"'<>]//g;
		
		push(@cook,$_);
	}
	return @cook;
}

#-----------------------------------------------------------
#  クッキー取得
#-----------------------------------------------------------
sub get_cookie {
	# クッキー取得
	my $cook = $ENV{HTTP_COOKIE};

	# 該当IDを取り出す
	my %cook;
	foreach ( split(/;/, $cook) ) {
		my ($key,$val) = split(/=/);
		$key =~ s/\s//g;
		$cook{$key} = $val;
	}

	# URLデコード
	my @cook;
	foreach ( split(/<>/, $cook{$cf{cookie_id}}) ) {
		s/%([0-9A-Fa-f][0-9A-Fa-f])/pack("H2", $1)/eg;
		s/[&"'<>]//g;

		push(@cook,$_);
	}
	return @cook;
}

#-----------------------------------------------------------
#  クッキー発行
#-----------------------------------------------------------
sub set_cookie {
	my ($key,$val,$del) = @_;

	# 時間定義
	my $gtime = $del eq 'del' ? time - 365*24*60*60 : time + 60*24*60*60;

	my ($sec,$min,$hour,$mday,$mon,$year,$wday,undef,undef) = gmtime($gtime);
	my @mon  = qw|Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec|;
	my @week = qw|Sun Mon Tue Wed Thu Fri Sat|;

	# 時刻フォーマット
	my $gmt = sprintf("%s, %02d-%s-%04d %02d:%02d:%02d GMT",
				$week[$wday],$mday,$mon[$mon],$year+1900,$hour,$min,$sec);

	print "Set-Cookie: $key=$val; expires=$gmt\n";
}

